package mail;

import javax.mail.*;
import javax.mail.internet.*;

import com.sun.mail.smtp.SMTPTransport;

import java.util.*;

/**
  * A simple email sender class. Can only send using smtp servers
  * which do not require SSL. Additional configuration needed to send it
  * to secure SMTP servers like gmail.
  */
public class SimpleSender
{

  /**
    * MAIN method to send a message given on the command line.
    * arg 0: smtpServer address
    * arg 1: to email address
    * arg 2: from email adress
    * arg 3: subject
    * arg 4: body
    */
  public static void main(String args[])
  {
    try
    {
      String smtpServer=args[0];
      String to=args[1];
      String from=args[2];
      String subject=args[3];
      String body=args[4];

      send(smtpServer, to, from, subject, body);
    }
    catch (Exception ex)
    {
      System.out.println("Usage: java com.lotontech.mail.SimpleSender"
       +" smtpServer toAddress fromAddress subjectText bodyText");
    }

    System.exit(0);
  }
  
  /**
   * Simple function for sending mail to the given email address. It only works
   * for servers which do not require any security check
   * @param smtpServer Name of the server through which the email will be sent
   * @param to Email address of the person to whom the email is being sent
   * @param from Email address of the person who is sending the email 
   * @param subject The subject line of the email
   * @param body The contents of the email
   */
  public static void send(String smtpServer, String to, String from
   , String subject, String body)
  {
    try
    {
      Properties props = System.getProperties();
      
      props.put("mail.smtp.starttls.enable","true");

      // -- Attaching to default Session, or we could start a new one --
      // -- Could use Session.getTransport() and Transport.connect()
      // , but assume we're using SMTP --

      props.put("mail.smtp.host", smtpServer);
      Session session = Session.getDefaultInstance(props, null);

      // -- Create a new message --
      Message msg = new MimeMessage(session);

      // -- Set the FROM and TO fields --
      msg.setFrom(new InternetAddress(from));
      msg.setRecipients(Message.RecipientType.TO,
        InternetAddress.parse(to, false));

      // -- We could include CC recipients too --
      // if (cc != null)
      // msg.setRecipients(Message.RecipientType.CC
      // ,InternetAddress.parse(cc, false));

      // -- Set the subject and body text --
      msg.setSubject(subject);
      msg.setText(body);

      // -- Set some other header information --
      msg.setHeader("X-Mailer", "LOTONtechEmail");
      msg.setSentDate(new Date());
      
//      SMTPTransport.
//      SMTPTransport.send(msg);
      // -- Send the message --
      Transport.send(msg);

      System.out.println("Message sent OK.");
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }
  }
}
