package swing;

import java.awt.Color;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Toolkit;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.WindowConstants;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

public class SwingUtility {

	public static final String BOXLAYOUT = "BoxLayout";
	
	/**
	 * Set layout of component to BoxLayout with the given orientation
	 * @param orientation Use 0 for BoxLayout.X_AXIS and 1 for BoxLayout.Y_AXIS
	 */
	public static void setBoxLayout(JComponent cp, int orientation) {
		if(orientation == 0)
			cp.setLayout(new BoxLayout(cp, BoxLayout.X_AXIS));
		else if(orientation == 1)
			cp.setLayout(new BoxLayout(cp, BoxLayout.Y_AXIS));
	}
	/**
	 * Sets the layout of the component to flowlayout.
	 * @param cp The component
	 * @param leading Whether to set the flowlayout leading property
	 */
	public static void setFlowLayout(JComponent cp, boolean leading) {
		if(leading)
			cp.setLayout(new FlowLayout(FlowLayout.LEADING));
		else
			cp.setLayout(new FlowLayout());
	}
	/**
	 * Surrounds the component with the given title
	 * @param panel The component
	 * @param title The title of the component
	 * @param makeTitleBold Specify whether the title should be bold or not
	 */
	public static void setBorderTitle(JComponent panel, String title, boolean makeTitleBold) {
		if(makeTitleBold == false)
			panel.setBorder(BorderFactory.createTitledBorder(title));
		else {
			Border b = BorderFactory.createLineBorder(Color.black);
	    	Font f = new Font(panel.getFont().getFontName(), Font.BOLD, panel.getFont().getSize());
			panel.setBorder(new TitledBorder(BorderFactory.createTitledBorder(b, title, TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION,f)));//""));
	    	
		}
	}
	/**
	 * Updates the font size of the component with the new value
	 * @param component The component
	 * @param newFontSize The new size
	 */
	public static void setComponentFontSize(JComponent component, int newFontSize) {
		Font f = new Font(component.getFont().getFontName(), component.getFont().getStyle(), newFontSize);
		component.setFont(f);    	
	}
	
	public static void setComponentFontStyle(JComponent component, int fontStyle) {
		Font f = new Font(component.getFont().getFontName(), fontStyle, component.getFont().getSize());
		component.setFont(f); 
	}
	
	   /** Returns width of the screen */
    public static int getScreenWidth() {
        return (int) Toolkit.getDefaultToolkit().getScreenSize().getWidth();
    }

    /** Returns height of the screen */
    public static int getScreenHeight() {
        return (int) Toolkit.getDefaultToolkit().getScreenSize().getHeight();
    }
    /**
     * Sets system dependent look and feel.
     */
    public static void setSystemDependentLookAndFeel(){
        try {            
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (UnsupportedLookAndFeelException e) {
            e.printStackTrace();
        }
    }

    /**
     * Function to display a custom panel in a dialog box.
     * @param frm The root frame
     * @param pnl The panel to be displayed
     * @param sTitle Title of the dialog box
     * @param btn Not sure
     */
    public static void showPanelInDialog(JFrame frm, JPanel pnl, String sTitle, JButton btn){
        JDialog dlg = new JDialog(frm, sTitle, true);
        dlg.getContentPane().add(pnl);
        dlg.getRootPane().setDefaultButton(btn);
        dlg.pack();
    
        dlg.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        dlg.setLocationRelativeTo(JOptionPane.getRootFrame());
        dlg.setSize(pnl.getSize());
        dlg.setLocationRelativeTo(null);
        
        dlg.invalidate(); dlg.validate(); dlg.repaint(); 
        dlg.setVisible(true);
    }

    /**
     * Hides and disposes the root dialog for the given panel  
     */
    public static void hideDialog(JPanel pnl){
        Container c = pnl.getParent();
        
        while (c != null) {
            if (c instanceof JDialog) break;
            c = c.getParent();
        }
        JDialog dg = (JDialog)c;
        
        if(dg != null)
        dg.dispose();
    }

}
