package swing.components;

import java.awt.Component;
import java.io.File;
import java.util.Vector;

import javax.swing.ComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;

/**
 * Displays two combo boxes in a row. Useful in constructing many
 * dialogs where a range of things is to be displayed.
 * @author Tandeep
 *
 *TODO: Support the 2 comboboxes in a column.

 */
class RangeDisplayer extends JPanel
 {
     JComboBox startCombo;
     JComboBox endCombo;
     
     JLabel labelBetweenCombos;
     JLabel labelBeforeCombos;
     
     /**
      * Create this component with the data provided
      * @param startComboData data for the first combo box
      * @param endComboData data for the second combo box
      */
     RangeDisplayer(Vector startComboData, Vector endComboData)
     {
    	 if(startCombo != null && endCombo != null) {
	         startCombo = new JComboBox(startComboData);
	         endCombo = new JComboBox(endComboData);
	     }else {
	    	 startCombo = new JComboBox();
	    	 endCombo = new JComboBox();
	     }
	    	 
    	 add(startCombo);
         add(endCombo);
     }
     /**
      * Set the renderer for the 2 combo boxes
      * @param renderer
      */
     public void setListCellRenderer(ListCellRenderer renderer) {
    	 startCombo.setRenderer(renderer);
    	 endCombo.setRenderer(renderer);
     }
     /**
      * Constructs this object with the data provided.
      * @param label The label that goes between the 2 comboboxes. Can be null
      * @param prefixLabel The label that goes before the 2 comboboxes. Can be null
      */
     RangeDisplayer(Object [] startComboData, Object []endComboData, String label, String prefixLabel)
     {
         startCombo = new JComboBox(startComboData);
         endCombo = new JComboBox(endComboData);
         labelBetweenCombos = new JLabel(label);
         labelBeforeCombos = new JLabel(prefixLabel);
         
         add(labelBeforeCombos);
         add(startCombo);
         add(labelBetweenCombos);
         add(endCombo);
     }
     
     /**
      * Either data can be null in which case the combo boxes are not updated.
      */
     public void updateData(Object [] startComboData, Object [] endComboData)
     {
         if(startComboData != null) 
         {
             startCombo.removeAllItems();
             
             for(int i = 0; i < startComboData.length; i++)
                 startCombo.addItem(startComboData[i]);
             
             if(startCombo.getItemCount() != 0)
                 startCombo.setSelectedIndex(0);	
         }
         if(endComboData != null) 
         {
             endCombo.removeAllItems();
             
             for(int i = 0; i < endComboData.length; i++)
                 endCombo.addItem(endComboData[i]);
             
             if(endCombo.getItemCount() != 0)
                 endCombo.setSelectedIndex(0);	
         }
         repaint();
     }
     /* (non-Javadoc)
      * @see javax.swing.JComponent#setVisible(boolean)
      */
     public void setVisible(boolean aFlag)
     {
         super.setVisible(aFlag);
     }
     /**
      * Returns array of size 2. 0th location contains item
      * selected from first combo and 1st location contains item
      * selected from second combo
      */
     public Object [] getRangeSelected()
     {
         Object [] range = new Object[2];
         range[0] = startCombo.getSelectedItem();
         range[1] = endCombo.getSelectedItem();
         
         return range;
     }
     /**
      * Utility function to return a listcellrenderer which given File data renderes,
      * just the name of that file. If the data is something else, probably will throw a classcastexception.
      */
	public static ListCellRenderer getFileNameDisplayerRenderer() {
		ListCellRenderer renderer = new ListCellRenderer() {

			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				if(value == null) return new JLabel("");
				String result = ((File)value).getName();
				return new JLabel(result);
			}};
			
			return renderer;
	}
 }
     