package swing.components;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputMethodEvent;
import java.awt.event.InputMethodListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.filechooser.FileFilter;

/**
 * Simple gui class to display a text field and a browse button. On-click browse button,
 * a filechooser dialog is displayed. 
 * @author Tandeep Sidhu
 */
public class TextFieldWithBrowseButton extends JPanel{
	JTextField field;
	JButton btn;
	/**
	 * Builds the gui. 
	 * @param fieldTxt Initial text for the text field. 
	 * @param browseBtnLabel Label for the browse button
	 * @param fileFilter Any file filter for the filechooser that comes up when browse button pressed
	 */
	public TextFieldWithBrowseButton(String fieldTxt, String browseBtnLabel, final FileFilter fileFilter) {
		field = new JTextField(fieldTxt);
		field.setColumns(20);
		btn = new JButton(browseBtnLabel);
		
		ActionListener action = new ActionListener() {
			
			public void actionPerformed(ActionEvent arg0) {
				JFileChooser chooser = new JFileChooser("C:");
				chooser.setFileFilter(fileFilter);
						
				int selection = chooser.showDialog(TextFieldWithBrowseButton.this, "Select");
				
				if(selection != chooser.CANCEL_OPTION) {
					field.setText(chooser.getSelectedFile().getAbsolutePath());
				}
			}
			
		};
		
		btn.addActionListener(action);
		setLayout(new FlowLayout(FlowLayout.LEADING));
		add(field);
		add(btn);
	}
	/**
	 * Builds the gui. 
	 * @param fieldTxt Initial text for the text field. 
	 * @param browseBtnLabel Label for the browse button
	 * @param fileFilter Any file filter for the filechooser that comes up when browse button pressed
	 * @param caretListener Useful to receive events from the text field
	 */
	public TextFieldWithBrowseButton(String fieldTxt, String browseBtnLabel, final FileFilter fileFilter, CaretListener caretListener) {
		field = new JTextField(fieldTxt);
		field.setColumns(20);
		btn = new JButton(browseBtnLabel);
		
		ActionListener action = new ActionListener() {
			
			public void actionPerformed(ActionEvent arg0) {
				JFileChooser chooser = new JFileChooser("C:");
				chooser.setFileFilter(fileFilter);
						
				int selection = chooser.showDialog(TextFieldWithBrowseButton.this, "Select");
				
				if(selection != chooser.CANCEL_OPTION) {
					field.setText(chooser.getSelectedFile().getAbsolutePath());
				}
			}
			
		};
		
		btn.addActionListener(action);
		field.addCaretListener(caretListener);
		setLayout(new FlowLayout(FlowLayout.LEADING));
		add(field);
		add(btn);
		
	}
	/**
	 * Returns the text of the text field
	 * @return
	 */
	public String getFieldText() {
		return field.getText();
	}
	public void setText(String txt) {
		field.setText(txt);
	}
}